
import sys
import json
import time
import threading
import socketserver
from Bot import BotTcpHandler


class ServerThread(threading.Thread):
    def __init__(self,arg):
        super(ServerThread, self).__init__()#注意：一定要显式的调用父类的初始化函数。
        self.arg=arg
    def run(self):#定义每个线程要运行的函数
        time.sleep(1)
        print (self.arg['ip'],":",self.arg['port'], " started.")
        self.arg["fun"](self.arg['ip'],self.arg['port'])



def loadConfig():
    ip='0.0.0.0'
    port=8180
    path=sys.path[0]
    with open(path+"/botConfig.json",'r') as jf:
        jstr=json.loads(jf.read())
    return jstr

def startOneServer(ip,port):
    try:
        server = socketserver.ThreadingTCPServer ((ip, port), BotTcpHandler)
        server.serve_forever()
    except  KeyboardInterrupt :
        print('i get en error')
        server.shutdown()
    except  OSError  as  ex:
        print(ex)

if __name__ == '__main__':
    confList= loadConfig()
    #print(confList)
    try:
        for k,v in confList.items():
            for bot in v["bot"]:
                arg={'fun':startOneServer,"ip":bot["ip"],"port":bot["port"]}
                st=ServerThread(arg);
                st.start()
    except Exception as e:
        print('aaaaaa',e)






#me: setup.py
# Author: ma
# Created Time:  2018-09-20
#############################################


from setuptools import setup, find_packages

setup(
    name = "ai_bot",
    version = "0.0.1",
    keywords = ("pip", "BotServer","Callcenter","VoiceBot","ai_bot","callcenter"),
    description = "18-ai_bot0.1.0",
    long_description = "ai_bot0.1.0",
    license = "MIT Licence",
    url = "https://github.com/MaPing01/pipProject",
    author = "MaPing01",
    author_email = "pingpingma001@163.com",

    packages = find_packages(),
    include_package_data = True,
    platforms = "any",
    install_requires = []
)




#coding:utf-8

import ESL
import json
import time
import datetime
import socketserver
import sys,urllib3,urllib
from MyDialog.d_payback import *
from urllib3.exceptions import InsecureRequestWarning

urllib3.disable_warnings(InsecureRequestWarning)
currentPath=sys.path[0]

'''
    web post请求
    url：请求地址，包括请求的参数由？和地址相连的参数，参数之间由&连接
    jsonBodyData：请求体，jsonstring
 '''
def webPostRequest(url, jsonBodyData):
    response = urllib3.PoolManager ().request ("POST", url, body=jsonBodyData.encode (),
                                               headers={'Content-Type': 'application/json'})
    return json.loads (response.data)

def webGetRequest(url):
    response = urllib3.PoolManager ().request("GET", url,headers={'Content-Type': 'application/json'})
    return json.loads (response.data)

def readJsonFile(filename):
    with open (filename, 'r') as load_f:
        return json.load (load_f)

class VoiceBase(object):
    def __init__(self,voice_param,*asrgs,**kwargs):
        self._voice_param=voice_param

    def initParam(self,con_str, **kwargs):
        p = ''
        for k, v in kwargs.items ():
            p += k + "=" + v + con_str
        return p

    @property
    def mader(self):
        if self._voice_param and "mader" in self._voice_param:
            return self._voice_param["mader"]
        return None

    @property
    def request_type(self):
        if self._voice_param and "request_type" in self._voice_param:
            return self._voice_param["request_type"]
        return None

    @property
    def param(self):
        if self._voice_param and "param" in self._voice_param:
            return  self.initParam("&",**self._voice_param["param"])+'auth=mifeng'
        return None

    @property
    def sdk_param(self):
        if self._voice_param and "param" in self._voice_param:
            return  self.initParam(",",**self._voice_param["param"])
        return None

    @property
    def url(self):
        if self._voice_param and "url" in self._voice_param:
            return self._voice_param["url"]
        return ""

    @property
    def body(self):
        if self._voice_param and "body" in self._voice_param:
            return self._voice_param["body"]
        return ""

class AsrHandle (VoiceBase):
    def __init__(self, voice_param,*args, **kwargs):
        super(AsrHandle,self).__init__(voice_param)

    @property
    def result_type(self):
        if self._voice_param and "result_type" in self._voice_param:
            return  self._voice_param["result_type"]
        return ""

    @property
    def sdk_param(self):
        tp=super (AsrHandle, self).sdk_param
        return  tp+ "start-input-timers=false,no-input-timeout=1000"

    def start(self, con, uuid, **kwargs):
        return con.execute ('detect_speech', 'zw_asr {' + self.sdk_param + '}yes_no yes_no', uuid)

    def pause(self, con, uuid):
        return con.execute ('detect_speech', 'pause', uuid)

    def resume(self, con, uuid):
        return con.execute ('detect_speech', 'resume', uuid)

    def detected(self, event):
        statu = event.getHeader ("Speech-Statu")
        body = event.getHeader ("_body")
        if statu == '5':
            if self.mader == "ali":
                return self.aliParse (body),True
            elif self.mader == "kd":
                return self.kdParse (body),True
            elif self.mader == 'bd':
                return self.bdParse (body),True
            else:
                print ("unknow asr type:" + self.mader)
        return None,False

    def bdParse(self, jsonTxt):
        print(' unimplement the method bd parse in asrhandle.')
        return None,None

    def kdParse(self, json_str):
        retStr = None
        if self.result_type == "json":
            ret = json.loads (json_str)
            for word in ret['ws']:
                for cword in word['cw']:
                    retStr += cword['w']
                    break
        else:
            retStr = json_str
        #print ("source:", json_str)
        #print ("target:", retStr)
        return None,None

    def aliParse(self, json_str):
        retStr = None
        if self.result_type == "json":
            ret = json.loads (json_str)
            if 'result' in ret:
                retStr = ret["result"]["text"]  # ,ret["result"]["sentence_id"]
        else:
            retStr = json_str
        return retStr

class TtsHandle (VoiceBase):
    @property
    def speak_type(self):
        if self._voice_param and "speak_type" in self._voice_param:
            return  self._voice_param["speak_type"]
        return None

    def say(self,con,uuid,ai):
        self.isPlaying = True
        if  self.speak_type=='tts':
            if self.request_type=='web':
                requestUrl=(self.url+'?'+self.param) % ai['text']
                con.execute ("playback", "shout:"+requestUrl, uuid)
            else:
                # con.execute ("speak", 'hello,boy', self.uuid)
                pass
        elif self.speak_type=='record':
            recordUrl=self.url+ai["recordNumber"]+".mp3"
            con.execute ("playback", recordUrl, uuid)
        else:
            print (self.speak_type, "is unknow play type,required tts or record(mp3,wav)")

class NlsHandle(VoiceBase):

    def __init__(self,voice_param,*asrgs,**kwargs):
        self.log_id=0
        self._end = False
        self._dialog=None
        super(NlsHandle,self).__init__(voice_param,*asrgs,**kwargs)

    @property
    def dialog_type(self):
        if  self._voice_param:
            return  self._voice_param["dialog"]["type"]
        return None

    @property
    def dialog(self):
        if   self._voice_param and not self._dialog:
            dialog_content=""
            dialog_p=self._voice_param["dialog"]
            if dialog_p["type"]=='buildin':
                fname=sys.path[0]+dialog_p["name"]
                dialog_content=readJsonFile(fname)
            else:
                dialog_content=webGetRequest(dialog_p["url"]+"?"+self.initParam("&",dialog_p["param"]))
            self._dialog=payback("",**dialog_content)
        return self._dialog



    '''
    对话交互
        如果结果是对话内容，此处则返回对话文本（tts）,录音文件名（record）
    '''
    def interop(self,userTxt='begin'):
        requestUrl=''
        if self.request_type=='web' and self.param:
            if self.dialog_type=="buildin":
                if userTxt == 'begin':
                    txt = self.dialog.context.current_dn
                    self._end = False
                else:
                    if userTxt != 'next' and userTxt != "GREET1" and userTxt != "GREET2" and userTxt != "GREET3":
                        requestUrl = (self.url + '?' + self.param)
                        if self.mader == "bd":
                            self.log_id += 1
                            self.body["log_id"] = self.log_id
                        jStr = json.dumps (self.body) % userTxt
                        jsonResponse = webPostRequest (requestUrl, jStr)
                        method, txt = self.parse (jsonResponse)
                    else:
                        if userTxt == 'next':
                            method = userTxt
                        else:
                            txt = userTxt
                    if method == 'end' or method == 'bye':
                        self._end = True
                    else:
                        if method:
                            txt= self.dialog.fetchNext(method)
                        elif txt:
                            txt= self.dialog.fetchQA(txt)
                        self._end=self.dialog.context.end
            else:
                print ("unimplement dialog")
        else:
            print("unimplement interop")
        return txt

    def parse(self,jsonTxt):
        if self.mader=='bd':
            return self.bdParse(jsonTxt)
        elif self.mader=='kd':
            return self.kdParse (jsonTxt)
        elif self.mader=='ali':
            return self.aliParse (jsonTxt)
        else:
            print (self.mader, "is unknow nls mader")

    def bdParse(self,jsonTxt):
        txt=None
        method=None
        if jsonTxt:
            if 'error_code' in jsonTxt  and jsonTxt['error_code']==0:
                txt=jsonTxt['result']['response']['action_list'][0]['say']# 1.0 需要编码解码s_str = s_ucode.encode ("utf-8")
                method=jsonTxt['result']['response']['action_list'][0]['custom_reply']
                if method and method.index('func'):
                        if 'func' in method:
                            method=json.loads(method)["func"]
                        else:
                            print (method)
                            method=None
            elif jsonTxt['error_code']!=0:
               print('error: ' , jsonTxt['error_code'],'error-code:',jsonTxt['error_msg'].encode ("utf-8"))
        return  method,txt

    def kdParse(self, jsonTxt):
        print(' unimplement the method kd parse in nlphandle.')
        return None,None

    def aliParse(self, jsonTxt):
        print (' unimplement the method ali parse in nlphandle.')
        return None,None

    @property
    def end(self):
        return self._end

'''
    如果nls返回的结果是语音流,则直接跳过nls过程,而将nls的相关参数配置到tts上
    tts可以直接把数据shout到fs中,播放出去
'''
class SampleBotTcpHandler(socketserver.BaseRequestHandler):
    def handle(self):
        if self.con.connected():
            self.setChannelVariable()
            while 1:
                event = self.con.recvEventTimed (100)
                self.SilenceTimeout()
                if event:
                    ename, euuid = self.getEventInfo (event)
                    print (ename)
                    if  euuid == self.uuid and ename in self.switch and self.switch[ename](event)==True:
                        break

        return

    def setup(self):
        self.rcontextUploadType='url'
        self.idleTime=None
        self.silenceTimes = 0
        self.silenceTimeout=10
        self._conf=None
        self.is_tnls=False
        reqInfo = self.request.fileno ()
        self.con = ESL.ESLconnection (reqInfo)
        self.getRequestInfo()
        self.subscriptionEvents()
        self.initSwitch()
        self.isAnswer=False
        self.isPlaying = False

    @property
    def config(self):
        if not self._conf:
            with open (sys.path[0] + "/botConfig.json", 'r') as jf:
               self._conf = json.loads (jf.read ())
        return  self._conf

    def finish(self):
        return

    def initSwitch(self):
        self.switch = {
                        "CHANNEL_ANSWER": self.evtAnswer, "CHANNEL_HANGUP": self.evtHangup,
                       "PLAYBACK_START": self.evtPlayStart, "PLAYBACK_STOP": self.evtPlayStop,
                       "DETECTED_SPEECH":self.evtDetectedSpeech, "CHANNEL_HANGUP_COMPLETE": self.evtHangupComplete,
                       "SERVER_DISCONNECTED":self.evtServerDisconnected
                       }

    def subscriptionEvents(self):
        self.con.events ("json", "all")
        self.con.sendRecv ('myevents')
        self.con.sendRecv ('divert_events')
        self.con.sendRecv ('linger')

    def getRequestInfo(self):
        info = self.con.getInfo ()
        self.uuid = info.getHeader ('unique-id')
        self.cname = info.getHeader ('Channel-Channel-Name')
        self.record_dir = info.getHeader ('variable_RecordDir')
        self.configName = info.getHeader ('variable_config_name')
        if self.config[self.configName]["config"]["isParamFrom"]!="config":
            #读取所有通道参数但前不支持
            self.asr = AsrHandle(json.loads(info.getHeader ('variable_asr_param')))
            self.tts = TtsHandle(json.loads (info.getHeader ('variable_tts_param')))
            self.nls = NlsHandle (json.loads (info.getHeader ('variable_nls_param')))
        else:
            cfg=self.config[self.configName]["config"]
            self.asr=AsrHandle(cfg["asr"])
            self.tts=TtsHandle(cfg["tts"])
            self.nls = NlsHandle (cfg["nls"])

    def setChannelVariable(self):
        #self.con.execute ('set', 'fire_asr_events=true', self.uuid)  #当无法接收到语音识别事件detected-speech时候可以将此行取消注释
        self.con.execute ('set', "rtp_enable_vad_out=true", self.uuid)
        #con.execute ("set", 'record_sample_rate=16000', self.uuid)
        self.con.execute ("set", 'variable_builder=mifeng', self.uuid)

    def getEventInfo(self,event):
        ename = event.getHeader ("Event-Name")
        euid= event.getHeader ('unique-id')
        #print( event.serialize ('json'))
        return  ename,euid

    def evtAnswer(self,event):
        self.isAnswer =True
        beginT=self.nls.interop('begin')
        self.asr.start (self.con, self.uuid)
        self.tts.say(self.con,self.uuid,beginT)
        self.nls.dialog.context.appendDialogContext (beginT, '')
        return False

    def evtDetectedSpeech(self,event):
        print("detected......",self.isPlaying)
        if self.asr and self.isPlaying == False:
            if self.asr.mader=='kd':
                #stop detected
                self.con.execute ('detect_speech', 'pause', self.uuid)
            asr_txt,isValid= self.asr.detected(event)
            if asr_txt and isValid==True:
                if self.nls:
                    txt = self.nls.interop(asr_txt)
                    self.nls.dialog.context.appendDialogContext(txt,asr_txt)
                    print ("人:",asr_txt)
                    print("                             机器:",txt["text"])
                else:
                    txt =asr_txt

                if self.tts :
                    #如果nls返回的结果为语音流,则在tts中直接调用,就可以把nls的相关信息直接配置到tts上
                    self.tts.say(self.con,self.uuid,txt)
        return self.nls.end #nls 返回结果为语音流暂未处理

    def evtPlayStart(self,event):
        self.isPlaying = True
        print("p start")
        self.idleTime = None
        return False

    def evtPlayStop(self,event):
        self.idleTime = datetime.datetime.now ()
        self.isPlaying = False
        print ("p end")
        if self.nls.end == True:
            self.con.execute ("hangup", "", self.uuid)
        if self.asr.mader == "ali":
            pass
        elif self.asr.mader == "kd":
            self.con.execute ('detect_speech', 'resume', self.uuid)
        else:
            self.con.execute ('detect_speech', 'resume', self.uuid)

        return False

    def evtHangup(self,event):
        return False

    def evtHangupComplete(self,event):
        self.uploadContext (self.con)
        self.uploadRecord (self.con)
        return True

    def evtServerDisconnected(self,event):
        return  True

    def SilenceTimeout(self):
        isTimeOut= False if not self.idleTime else (datetime.datetime.now() - self.idleTime).seconds > self.silenceTimeout
        newSay=None
        if self.isAnswer==True and (not self.isPlaying)  and isTimeOut==True:
            if self.silenceTimes==0:
                newSay= self.nls.interop('GREET1')
            elif self.silenceTimes==1:
                newSay = self.nls.interop ('GREET2')
            elif self.silenceTimes==2:
                newSay = self.nls.interop ('GREET3')
            else:
                text = self.nls.interop ('next')
                if text:
                    newSay=text
            if newSay and self.isPlaying == False:
                self.tts.say (self.con,self.uuid,newSay)
                self.nls.dialog.context.appendDialogContext (newSay, '')
            self.silenceTimes+=1

    def saveDailogContext(self):
        filename = sys.path[0]+'/dialogRecord/'+self.uuid+'.json'
        dcontexts=self.nls.dialog.context.getDialogContexts()
        rDcontext=[]
        for text in dcontexts:
            if text['man'] and text['man']!='':
               man=text['man']
            else :
                man=""
            if text['ai']:
                ai=text['ai']['text']
            else:ai=""
            if man or ai:
                rDcontext.append({"man":man,'ai':ai})
        with open (filename, 'w') as f:  # 如果filename不存在会自动创建， 'w'表示写数据，写之前会清空文件中的原有数据！
            f.write (json.dumps(rDcontext).encode('utf-8').decode('unicode_escape'))
        return filename

    def uploadContext(self, con):
        print ("已经挂断,保存对话内容")
        contextfile = self.saveDailogContext ()
        print ('保存完成，上传对话内容')
        apisrt =self.config[self.configName]["dialogrecord_url"]+"" + self.uuid
        if self.rcontextUploadType == 'fs':
            self.fsUpload (con, contextfile, apisrt)
        else:
            self.urlUpload (apisrt, contextfile)
        print ('上传对话内容完成，')

    def fsUpload(self, con, file, url):
        rcf = " RecordContextFile=" + file
        uplaodstr = url + rcf + ' %s nopost event'
        evt = con.api ('curl_sendfile ' + uplaodstr)
        print (evt.serialize ('json'))

    def urlUpload(self, url, file):
        with open (file) as cjson:
            file_data = cjson.read ()
        for i in range (3):
            response = urllib3.PoolManager ().request ("POST", url, fields={'RecordContextFile': (file, file_data)})
            rd = json.loads (response.data)
            if response.status == 200:
                break
            time.sleep (5)
        return rd

    def uploadRecord(self, con):
        print ('保存完成，上传对话录音')
        apisrt = self.config[self.configName]["dialogrecord_url"]+"%s %s nopost event"
        rf = "RecordFile=" + self.record_dir + "/%s.wav"
        uplaodstr = apisrt % (self.uuid, (rf % self.uuid))
        evt = con.api ('curl_sendfile ' + uplaodstr)
        print (evt.serialize ('json'))
        print ('上传对话录音完成')
        #print (uplaodstr)
        return False

server = socketserver.ThreadingTCPServer (('0.0.0.0', 8240), SampleBotTcpHandler)
server.serve_forever()


#coding:utf-8

import ESL
import  json
import time
import  base64
import  datetime
import threading
import socketserver
import sys,urllib3,urllib
from urllib3.exceptions import InsecureRequestWarning

# 13772491757  祝  13916483739 张 15062405910  李 18209288679
class PhoneDialer(object):
    '''
    外呼拨号器
    '''
    def __init__(self,task,line,bot,**kwargs):
        '''
        初始化拨号器
        :param task:
        :param line: 外线配置对象
        '''
        self.line=line;
        self.bot=bot
        self.task=task  # 呼叫号码
        self.t_conf=kwargs
        self.dail_str = 'originate {param}sofia/gateway/{linename}/{dest} {bot} xml {dialplan}' #拨叫字符串
        self.isEslConnected = False #esl连接状态

    def getEsl(self):
        '''
        获取esl连接对象
        :return: 返回连接成功的esl对象
        '''
        con=ESL.ESLconnection(self.line['serverIp'],self.line['serverPort'],self.line['clientName'])
        if con.connected():
            con.events('json','all')
            self.isEslConnected=True
        else:
            #此处需要引发连接连接服务器异常
            pass
        return  con

    def stopEsl(self,con):
        '''
        断开esl连接
        :param con: esl连接对象
        :return: 无返回
        '''
        if  self.isEslConnected==True:
            con.disconnect()
            self.isEslConnected=False

    def dial(self,number,is_supervision=False,websock=None):
        '''
        拨号，websock不空的话就会监视拨打状态，否则不监视
        :param websock: websock对象
        :return:  无返
        '''
        con=self.getEsl()
        #param ="{originattion_caller_id_number="+self.line['aiPhoneNum']+",config_name=testbot,callee_id_number="+number["Phone"]+",Scence_id="+self.task['scencenumber']+",Scence_name="+self.task['scencename']+"}"
        if self.t_conf['rest_type']=='java':

            if 'companycode' in self.task:
                owner=str(self.task['companycode'])
            else:
                owner=str(number['companycode'])
            case=number['casenumber'] if number['casenumber'] else ''
            mone=number['overdueamount'] if number['overdueamount'] else 0
            param = "{companycode="+owner+\
                    ",originattion_caller_id_number=" + self.task['bot']['number'] + \
                    ",config_name=testbot,callee_id_number=" +  number["phone"] + \
                    ",Scence_id=" + self.task['scence']['code'] +\
                    ",Scence_name=" + self.task['scence']['name'].upper() +\
                    ",scence_obj_id="+ self.task['scence']['id']+\
                    ",bot_obj_id="+self.task['bot']['id']+\
                    ",line_obj_id="+self.task['line']['id']+ \
                    ",creater_id="+self.task["createrid"]+\
                    ",customer_obj_id="+number["id"]+\
                    ",money="+str(mone)+\
                    ",c_name="+number['name']+\
                    ",case_number="+case+"}"
        else:
            param = "{owner_id="+str(self.task['creater']['id'])+\
                    ",originattion_caller_id_number=" + self.task['bot']['number'] + \
                    ",config_name=testbot,callee_id_number=" +  number["phone"] + \
                    ",Scence_id=" + self.task['scence']['code'] + \
                    ",Scence_name=" + self.task['scence']['name'] .upper()+ \
                     ",scence_obj_id="+ self.task['scence']['id']+\
                    ",bot_obj_id="+self.task['bot']['id']+\
                    ",line_obj_id="+self.task['line']['id']+\
                    ",customer_obj_id="+number["id"]+"}"

        ds=self.dail_str.format(param=param,linename=self.line['linename'],dest=number["phone"],bot=self.task['bot']['number'],dialplan=' default')
        print("dial:",ds)
        con.api(ds)
        if  is_supervision == True:
            self.supervision(con,number,websock)
        self.stopEsl (con)

    def supervision(self,con,number,webock):

        print('supervision', number )
        '''
        监视通话事件
        :param con:esl连接对象
        :param webock: websocket 对象
        :return: 无返回
        '''
        while 1:
            evt = con.recvEvent ()
            if evt:
                evt_name=evt.getHeader ("Event-Name")
                if evt_name == "SERVER_DISCONNECTED":
                    print('号码:', number['phone'], 'disconnected')
                    break
                if   evt_name=='CHANNEL_HANGUP_COMPLETE':
                    print('号码:', number['phone'], 'hangup complete')
                if evt_name == "CHANNEL_DESTROY" :
                    print('号码:',number['phone'],'destroy')
                    break
                if evt_name == 'CHANNEL_ANSWER':
                    print("号码:",number['phone'],'answered')
                    self.sendMessagToClient (evt_name,number, webock, evt)
                if evt_name == 'CHANNEL_HANGUP':
                    print('号码:',number['phone'],'hangup')
                    self.sendMessagToClient(evt_name,number,webock,evt)

    def setDest(self,dest):
        pass

    def sendMessagToClient(self,evt_name,number,websock,fsevent):
        '''
        往客户端会送电话状态，通过websocket
        :param evt_name: 事件名称
        :param websock: websock实例
        :param fsevent: fs socket event
        :return: 无返回
        '''
        act = fsevent.getHeader ('Answer-State')
        target = fsevent.getHeader ('Caller-Caller-ID-Number')
        cs = fsevent.getHeader ('Channel - State')
        ccs = fsevent.getHeader ('Channel-Call-State')
        cause = fsevent.getHeader ('Hangup-Cause')
        if  websock:
            msg=json.dumps (
                {'phone-number': target,
                 'action': evt_name,
                 'answer-state': act if act else "",
                 'channel-state': cs if cs else "",
                 'channel-callstate': ccs if ccs else "",
                 'hangup-hause': cause if cause else ""})
            websock.send(msg.encode('utf-8'))
        else:
            self.updateNumber3Times(number['id'],act if act else "finish")

    def webRequest(self,method, url, body="", auth="Basic"):
        print("webRequest",url,body)
        try:
            if body:
                response = urllib3.PoolManager ().request (method, url, body=body.encode (),
                                                           headers={'Content-Type': 'application/json',
                                                                    'Authorization': auth})
            else:
                response = urllib3.PoolManager ().request (method, url, headers={'Content-Type': 'application/json',
                                                                                 'Authorization': auth})
            if response.status == 200 :
                #print(response.data)
                return json.loads (response.data)
        except ConnectionRefusedError:
            print('服务器拒绝连接:',url)
        return None

    def getAuthStr(self):
        upwd = self.t_conf['username'] + ":" + self.t_conf['password']
        return "Basic " + base64.standard_b64encode (upwd.encode ()).decode ()

    def updateTask(self,taskId, statu):
        newtask={}
        if statu=='excuting':
            start_time=datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
            newtask['Status']=statu
            newtask['StartTime']=start_time
        elif statu=='finish':
            end_time=datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
            newtask['Status']= statu
            newtask['EndTime']=end_time
        else:
            newtask={"Status": statu}

        if self.t_conf['rest_type'] == 'java':
            url = self.t_conf['tasks_update_url']
            newtask['id'] = taskId
            jsstr = json.dumps (newtask).lower()
        else:
            url = self.t_conf['tasks_update_url'] + "%s/" % taskId
            jsstr = json.dumps (newtask)

        return self.webRequest ("patch", url, jsstr, self.getAuthStr ())

    def updateNumber(self,number,statu):
        if   self.t_conf['rest_type']=='java':
            url = self.t_conf["customer_update_url"]
            bd = json.dumps ({"id":number,"dialstatu": statu}).lower()
        else:
            url = self.t_conf["customer_update_url"] + "%s/" % number
            bd = json.dumps ({"DialStatu": statu})
        return self.webRequest("patch",url,bd,self.getAuthStr())

    def updateNumber3Times(self,number,statu):
        update_count = 0
        while not  self.updateNumber(number,statu):
            if  update_count<3:
                time.sleep(30)
                update_count+=1
            else:
                return  False
        return  True


    def updateTask3Times(self,taskId,statu):
        update_count = 0
        while  not self.updateTask (taskId,statu):  # 更新任务的状态为开始执行
            if update_count<3:
                time.sleep (30)
                update_count += 1
            else:
                return False
        return True

    def dials(self):
        if self.updateTask3Times(self.task["id"], 'excuting')==True:
            for number in self.task["numbers"]:
                    try :
                        if number["dialstatu"] == "ready" or number["dialstatu"] == "new" :
                            if  self.updateNumber3Times(number['id'],"calling") :#更新号码状态
                                self.dial(number,True)
                            else:
                                print('号码状态失败:',number['phone'])
                    except  Exception as exp:
                         print("Exception in dials metohed of  dialer",number['phone'],"exp:",exp.args)
                #self.updateNumber (number['Id'], "")  # 更新号码状态
            # 更新任务的状态
            self.updateTask3Times(self.task["id"], "finish")





#coding:utf-8

import  sys
import  AutoCall.AutoDialer as adialer



#adialer.run(sys.path[0]+"/cc.conf")

adialer.run_with_queue(sys.path[0]+"/cc.conf")
#adialer.run_with_queue(sys.path[0]+"/cc_test.conf")


#coding:utf-8

import ESL
import json
import datetime
import time
import sys,urllib3,urllib
import socketserver
from urllib3.exceptions import InsecureRequestWarning
import ssl
import  threading
import xml.dom.minidom as mdom
import queue
import  gc
from MyDialog.d_payback import *

urllib3.disable_warnings(InsecureRequestWarning)

recordContextPath=sys.path[0]

class BotTcpHandler(socketserver.BaseRequestHandler):
    access_token = '24.8a901b663eff65bbf4703ab27e156861.2592000.1538309173.282335-11126101'
    rcontextUploadType="url"
    uint_version = 2
    asr_mader="ali"   #ali 阿里巴巴,bd 百度,kd 科大讯飞
    asr_resutlType= "json"#'plain'#'xml'#
    def InitHandle(self):
        self.scence = '9720'#'2490'#'3400'
        self.switch = {"CHANNEL_ANSWER": self.EventAnswer, "CHANNEL_HANGUP": self.EventHangup,
                       "PLAYBACK_START": self.EventPlayStart, "PLAYBACK_STOP": self.EventPlayStop,
                       "DETECTED_SPEECH": self.EventDetectedSpeech,"CHANNEL_HANGUP_COMPLETE":self.EventHangupComplete,
                       "CHANNEL_DESTROY":self.distroy
                       }
        self.unit_log_id = 0
        self.silenceTimeout = 5
        self.silenceTimes = 0
        self.isAnswer = False  # 是否应答
        self.isPlaying = False  # 机器是否正在说话
        self.asrIdlStartTime = None  # 语音识别，静音计时开始时间
        self.bot_session = ''
        #self.playQueue = None#queue.Queue()
        self.sentence_id=None
        self.isBreak = False
        self.temp=0
        self.suffix = '.mp3'
        #self.sayThread=None

    def GetEventInfo(self,event):
        ename = event.getHeader ("Event-Name")
        e_sequence = event.getHeader ("Event-Sequence")
        euid= event.getHeader ('unique-id')
        #print( event.serialize ('json'))
        return  ename,euid

    def GetChannelInfo(self,con):
        info = con.getInfo ()
        #print (info.serialize ('json'))
        self.uuid = info.getHeader ('unique-id')
        self.cname = info.getHeader ('Channel-Channel-Name')
        self.recordDir=info.getHeader('variable_RecordDir')
        self.scence =info.getHeader('variable_Scence_id')
        self.case=info.getHeader("variable_case_number")
        self.money = info.getHeader("variable_money")
        self.c_name = info.getHeader("variable_c_name")
        self.scenceName = info.getHeader ('variable_Scence_name')
        self.userInfo={"name":"ldd","sex":"male","alias":"li_m","money":2320.,"over_time":5}
        print("scenece:",self.scence,self.scenceName)

    def SetChannelVariable(self,con):
        con.execute ('set', 'fire_asr_events=true', self.uuid)  #当无法接收到语音识别事件detected-speech时候可以将此行取消注释
        con.execute ('set', "rtp_enable_vad_out=true", self.uuid)
        #con.execute ("set", 'record_sample_rate=16000', self.uuid)
        con.execute ("set", 'variable_builder=mifeng', self.uuid)
        con.execute("set","playback_delimiter=@",self.uuid)
        con.execute("set", "playback_sleep_val=50", self.uuid)

    def uploadContext(self,con):
        print ("已经挂断,保存对话内容")
        contextfile = self.saveDailogContext ()
        print ('保存完成，上传对话记录')
        apisrt = "http://116.236.220.214:22286/etc-web/v1/api/dialog/updateDialog?uuid="+self.uuid
        #apisrt = "http://114.116.39.230:8500/VoiceBotService/v1/botManager/dialogRecord?uuid=" + self.uuid
        if BotTcpHandler.rcontextUploadType=='fs':
            self.fsUpload(con,contextfile,apisrt)
        else:
            self.urlUpload(apisrt,contextfile)
        print ('上传对话记录完成，')

    def fsUpload(self,con,file,url):
        rcf = " RecordContextFile="+file
        uplaodstr = url+rcf+' %s nopost event'
        evt = con.api ('curl_sendfile ' + uplaodstr)
        print (uplaodstr)
        #print (evt.serialize ('json'))

    def urlUpload(self,url,file):
        try:
            with open(file) as cjson:
                file_data=cjson.read()
                print(file," file length：",len(file_data))
        except FileNotFoundError as fnfe:
            print(datetime.datetime.now(),'文件：',file,'无法找到' )
            return None

        for i in  range(3):
            response = urllib3.PoolManager().request("Patch", url,
                                                     fields={'recordcontextfile': (file, file_data),
                                                                 "score": self.payback.context.score,
                                                                 "case_number":self.case,
                                                                "attitude": self.payback.context.attitude[0]})
            #response = urllib3.PoolManager ().request ("Patch", url,fields={'RecordContextFile': (file, file_data),"Score":self.payback.context.score,"Remark":self.payback.context.attitude})
            rd= json.loads (response.data)
            if response.status==200:
                break
            time.sleep(5)
        return rd

    def urlUploadRecord(self,url,file):
        try:
            with open(file,"rb") as cjson:
                file_data=cjson.read()
                print(file, " file length：", len(file_data))
        except FileNotFoundError as fnfe:
            print(datetime.datetime.now(),'文件：',file,'无法找到' )
            return  None

        for i in  range(3):
            response = urllib3.PoolManager().request("Patch", url, fields={'recordfile': (file, file_data)})
            #response = urllib3.PoolManager ().request ("Patch", url,fields={'RecordFile': (file, file_data)})
            rd= json.loads (response.data)
            if response.status==200:
                break
            time.sleep(5)
        print(rd)
        return rd

    def uploadRecord(self,con):
        print (datetime.datetime.now(),'保存完成，上传对话录音')

        if BotTcpHandler.rcontextUploadType == 'fs':
            # apisrt = "http://114.116.39.230:8500/VoiceBotService/v1/botManager/dialogRecord?uuid=%s %s nopost event"
            apisrt = "http://116.236.220.214:22286/etc-web/v1/api/dialog/updateDialog?uuid=%s %s nopost event"
            rf = " RecordFile=" + self.recordDir + "/%s" + self.suffix
            uplaodstr = apisrt % (self.uuid, (rf % self.uuid))
            evt = con.api('curl_sendfile ' + uplaodstr)
            bd = evt.getHeader('_body')
            if  bd and '200' in bd or 'Ok' in bd or bd['retMsg']=='ok':
                print(datetime.datetime.now(),'上传对话记录录音完成')
            else:
                print(datetime.datetime.now(),'录音完成上传失败')
            print(uplaodstr)
        else:
            f= self.recordDir+"/"+self.uuid+self.suffix
            urlr="http://116.236.220.214:22286/etc-web/v1/api/dialog/updateDialog?uuid="+self.uuid
            urlrd=self.urlUploadRecord(urlr,f)
            print(datetime.datetime.now(),"上传返回:",urlrd)
            if  urlrd and ('200' in urlrd or 'Ok' in urlrd or urlrd['retMsg']=='ok'):
                print(datetime.datetime.now(),'上传对话记录录音完成')
            else:
                print(datetime.datetime.now(),'录音完成上传失败')
            print(urlr)

        return  False

    def EventHangupComplete(self,con,event):
        print('挂断后续工作处理完成，开始上传通话记录与录音')
        self.uploadContext(con)
        self.uploadRecord(con)
        return  True

    def distroy(self,con):
        print("通道销毁")
        return  True

    def handle(self):
        self.InitHandle()
        reqInfo = self.request.fileno()
        con = ESL.ESLconnection (reqInfo)
        self.sentence_id=None;
        self.GetChannelInfo (con)
        self.payback = payback (recordContextPath+'/dialogflow/'+self.scenceName+'.json')
        con.events ("json", "all")
        con.sendRecv ('myevents')
        con.sendRecv('divert_events')
        con.sendRecv('linger')
        if con.connected():
            print ("已经连接成功")
            #self.sayThread=threading.Thread (target=self.threadSay,args=(con,))
            #self.sayThread.start ()
            #print("说话线程已启动")
            self.SetChannelVariable(con)
            self.EventHandle(con)
            #self.myAsrModEvnetHandle(con)
        #self.sayThread.join()
        con.disconnect ()

    def saveDailogContext(self):
        filename = recordContextPath+'/dialogRecord/'+self.uuid+'.json'
        dcontexts=self.payback.context.getDialogContexts()
        rDcontext=[]
        for text in dcontexts:
            if text['man'] and text['man']!='':
               man=text['man']
            else :
                man=""
            if text['ai']:
                ai=text['ai']['text']
            else:ai=""
            if man or ai:
                rDcontext.append({"man":man,'ai':ai})
        with open (filename, 'w') as f:  # 如果filename不存在会自动创建， 'w'表示写数据，写之前会清空文件中的原有数据！
            f.write (json.dumps(rDcontext).encode('utf-8').decode('unicode_escape'))
        return filename

    def myAsrModEvnetHandle(self,con):
        print("我的asr测试开始")
        con.execute("answer","",self.uuid)
        con.execute('detect_speech','zw_asr {start-input-timers=false,no-input-timeout=1000}yes_no yes_no', self.uuid)
        #con.execute ("playback", "/root/workspace/zw_project/VoiceBot/record/beieronline/FAQ11.mp3", self.uuid)
        while 1:
            event =con.recvEvent()
            if event:
                ename=event.getHeader ("Event-Name")
                #print(event.serialize())
                if ename=='DETECTED_SPEECH':
                    statu = event.getHeader ("Speech-Statu")
                    body = event.getHeader ("_body")
                    print("statu:",statu,"  r:",body)
                    if statu=='13' or statu=='8' or statu=='16' or statu=='5':
                        print('resume')
                if ename=="SERVER_DISCONNECTED":
                    break

    def aliDetectedSpeech(self,con,event):
        statu = event.getHeader ("Speech-Statu")
        body = event.getHeader ("_body")
        #print("ali return:",body," statu:",statu)
        if statu == '5'   :
            uSay,s_id=self.parseJson(body)
            print("ali detected(", datetime.datetime.now(), ")", uSay,s_id)
            if  self.sentence_id!=s_id and uSay:
                self.sentence_id=s_id
                #print('is playing ',self.isPlaying)
                if self.isPlaying == False:
                    aiSay = self.fetchSayTxt (con, uSay)
                    #print ("uSay:", uSay)
                    if aiSay :
                        self.sayTo(con,aiSay,uSay)
                        self.isBreak = False
                elif self.isPlaying == True and  self.isBreak==False:
                    print('打断下：')
                    self.isBreak = True
                    con.api('uuid_fileman '+self.uuid+' pause')
                    aiSay = self.fetchSayTxt (con, uSay)
                    if aiSay :
                        print('停止说话')
                        con.api('uuid_fileman '+ self.uuid+' stop' )
                        self.sayTo(con,aiSay,uSay)
                    else:
                        con.api('uuid_fileman '+self.uuid+' pause')

        return  False

    def kdDetectedSpeech(self,con,event):
        con.execute ('detect_speech', 'pause', self.uuid)
        statu = event.getHeader ("Speech-Statu")
        body = event.getHeader ("_body")
        print ("kd Detected time:", datetime.datetime.now ())
        if statu == '5':
            uSay=self.kd_parseJson(body)
            if uSay and self.isPlaying == False:
                aiSay = self.fetchSayTxt (con, uSay)
                #print ("对话处理完 time:", datetime.datetime.now ())
                if  aiSay:
                    self.sayTo (con, aiSay, uSay);

    def kd_parseJson(self,jsonStr):
        strRet = '';
        if self.asr_resutlType=="json":
            ret = json.loads (jsonStr)
            for word in ret['ws']:
                for cword in word['cw']:
                    strRet+=cword['w']
                    break
        else:
            strRet =jsonStr
        print ("source:",jsonStr)
        print ("target:",strRet)
        return  strRet,None

    def parseJson(self,jsonStr):
        retStr=''
        sentence_id=None
        #print(jsonStr)
        if self.asr_resutlType=="json":
            ret = json.loads (jsonStr)
            if 'result' in ret:
                retStr=  ret["result"]["text"]
                sentence_id=ret["result"]["sentence_id"]
        else:
            retStr =jsonStr
        return retStr,sentence_id

    def EventHandle(self,con):
        print (self.uuid,"应答，开始对话")
        #self.EventAnswer(con,None)
        #con.execute ("playback", "/root/workspace/zw_project/VoiceBot/record/beieronline/FAQ11.mp3", self.uuid)
        while 1:
            event=con.recvEvent()
            if event:
                ename,euuid=self.GetEventInfo(event)
                #print("event:",ename)

                if (ename in self.switch) and self.switch[ename](con,event) == True:
                    break
                if ename=='SERVER_DISCONNECTED':
                    break
            self.SilenceTimeout(con)
        print('通话结束，事件监退出')

    def EventDetectedSpeech(self, con, event):
        self.unit_log_id += 1
        #self.asrIdlStartTime=None
        if self.asr_mader=="ali":
            return  self.aliDetectedSpeech(con,event)
        elif self.asr_mader=='bd':
            return  self.bdDetectedSpeech(con,event)
        elif self.asr_mader=="kd":
            return self.kdDetectedSpeech(con,event)
        return  False

    def bdDetectedSpeech(self,con,event):
        speechType = event.getHeader ("Speech-Type")

        if speechType == 'begin-speaking':
            print ('我听到有人在说话:')
        elif speechType == 'detected-speech':
            self.asrIdlStartTime=None
            asrResult=event.getHeader('_body')
            if asrResult and (not (asrResult == 'Completion-Cause: 002')):
                if self.isPlaying:
                    con.api('uuid_break ' + self.uuid)
                    self.isPlaying=False
                userSay=self.parseXml(asrResult)
                con.execute ("detect_speech", 'pause', self.uuid)
                aiSay=self.fetchSayTxt(con,userSay)
                if aiSay:
                    self.say (con,userSay)
                else:
                    con.execute ('detect_speech', 'resume', self.uuid)
        return False

    def EventHangup(self,con,event):
        print("挂断")
        return False

    def SilenceTimeout(self,con):
        isTimeOut= False if not self.asrIdlStartTime else (datetime.datetime.now() - self.asrIdlStartTime).seconds > self.silenceTimeout
        newSay=None
        if self.isAnswer==True and (not self.isPlaying)  and isTimeOut==True:
            if self.silenceTimes==0:
                #self.silenceTimeout=3
                newSay= self.payback.fetchQA ('GREET1')
            elif self.silenceTimes==1:
                #self.silenceTimeout = 2
                newSay = self.payback.fetchQA ('GREET2')
            elif self.silenceTimes==2:
                newSay = self.payback.fetchQA ('GREET3')
            else:
                text = self.fetchSayTxt (con)
                if text:
                    newSay=text
            if newSay and self.isPlaying == False:
                #self.inQueue(newSay)
            #if self.isPlaying == False:
                self.sayTo (con,newSay,"")
            self.silenceTimes+=1

    def EventAnswer(self,con,event):
        print("answered")
        if self.asr_mader=='ali':
            con.execute ('detect_speech', 'zw_asr {start-input-timers=false,no-input-timeout=1000}yes_no yes_no',  self.uuid)
        else :
            con.execute ('detect_speech','unimrcp {start-input-timers=false,no-input-timeout=1000}yes_no yes_no', self.uuid)
        #self.inQueue(self.payback.context.current_dn)
        #if self.isPlaying == False:
        self.sayTo (con,self.payback.context.current_dn,"")
        self.isAnswer=True
        return False

    def EventPlayStart(self,con,event):
        #print ("play start")
        #print(event.serialize('json'))
        self.isPlaying = True
        self.asrIdlStartTime =None
        #print ("play start time:", datetime.datetime.now (),self.isPlaying)
        #print (('PlayStart new', datetime.datetime.now()))
        return False

    def EventPlayStop(self,con,event):
        #print ("play end time:", datetime.datetime.now ())
        #print(event.serialize('json'))
        self.asrIdlStartTime = datetime.datetime.now ()
        if self.asr_mader=="ali":
            # last_offset= event.getHeader("variable_playback_last_offset_pos")
            # samples=event.getHeader("variable_playback_samples")
            # print("last_offset:",last_offset,"samples:",samples)
            # if samples!=last_offset:
            #     self.isBreak=True
            # else:
            #     self.isBreak=False
            pass
        elif self.asr_mader=="kd":
            con.execute ('detect_speech', 'resume', self.uuid)
        else:
            con.execute ('detect_speech', 'resume', self.uuid)
        self.isPlaying = False
        if self.payback.context.end == True:
            con.execute("hangup","",self.uuid)
        return False

    def parseXml(self,xmlStr):
        '''
        解析语音识别结果，获取说话内容
        :param xmlStr: 语言识别的ｘｍｌ内容
        :return: 说话内容
        '''
        if xmlStr:
            xdom=mdom.parseString (xmlStr)
            nod= xdom.documentElement.getElementsByTagName('input')
            #print (nod[0].childNodes[0].nodeValue)
            if nod[0] and nod[0].childNodes[0] and nod[0].childNodes[0].nodeValue:
                strtext= nod[0].childNodes[0].nodeValue
                return strtext[0:-1]
        return None

    def fetchSayTxt(self, con, txtFrom = "next"):
        if BotTcpHandler.uint_version==2:
            toTxt="AN1"
            method = ""
            toTxt, method = self.interop2(txtFrom)
            #print("unit return,txt:",toTxt,'method:',method)
            if method:
                toTxt = self.payback.fetchNext (method)
            elif toTxt :
                if toTxt=="AN1":
                    toTxt =self.payback.fetchNext('next')
                else:
                    toTxt =self.payback.fetchQA(toTxt)
            else:
                toTxt=None
        return toTxt

    '''
    def inQueue(self,aiSay):
        print("in queue:",aiSay)
        self.playQueue.put_nowait (aiSay)
    
    def threadSay(self,con):
        while 1:
            if not self.playQueue.empty():
                aiSay=self.playQueue.get_nowait();
                print ('out queue:',aiSay)
                self.sayTo(con,aiSay,'')
            else:
                time.sleep(1)
        return

    def say(self,con,utext=""):
        while  not self.playQueue.empty():
            aiSay=self.playQueue.get_nowait();
            print ('out queue:',aiSay)
            self.sayTo(con,aiSay,utext)
            print ('palying:')
        print("return :")
        return
    '''

    def sayTo(self,con,text,utext=""):
        if not text: #将无法获取unit结果的对话做不理会处理。
            return ;
        self.isPlaying=True
        print ('@人说:','(',datetime.datetime.now(),')', utext,self.payback.context.score)
        print ('                        @机器说:','(',datetime.datetime.now(),')', text['text'])
        self.payback.context.appendDialogContext (text, utext)
        if text["type"]=="tts":
            tts ='%s%s%s' % ("shout://tsn.baidu.com/text2audio?lan=zh&ctp=1&cuid=abcdxxx&tok="+BotTcpHandler.access_token+"&tex=" , text['text'].replace(" ",""), "&vol=9&per=0&spd=6&pit=3")
            con.execute ("playback", tts, self.uuid)
        elif text["type"]=="record":
            pb_str=self.get_playback_str(sys.path[0]+'/record/'+self.scenceName+"/",text["recordNumber"])
            if pb_str:
                con.execute ("playback", pb_str, self.uuid)
        else:
            print(text["type"],"is unknow play type,required tts or record(mp3,wav)")

    def interop2(self,text,userid='mifeng'):
        if text == "next":return '', text
        url='https://aip.baidubce.com/rpc/2.0/unit/bot/chat?access_token=' + self.access_token
        client_session={ "client_results":"", "candidate_options":[]}
        request={ "bernard_level":0,"query":text,"updates":"", "user_id":userid,
                  "query_info":{"asr_candidates":[],"source":"KEYBOARD","type":"TEXT"},
                  "client_session":json.dumps(client_session)}
        pd={'bot_session':self.bot_session,'log_id':self.unit_log_id,'request':request,"bot_id":self.scence,"version":"2.0"}
        content =self.urlRequest(url,pd)
        aistr = ''
        aimeth = ''
        if (content):
            if 'error_code' in  content and content['error_code']==0:
                self.bot_session='' #self.bot_session = content['result']['bot_session']
                aisay=content['result']['response']['action_list'][0]['say']
                aimethod=content['result']['response']['action_list'][0]['custom_reply']
                #if 'func' in aimethod :
                    #mdist = json.loads(aimethod)
                    #aimethod =mdist['func']
                if aisay:
                    aistr = aisay #s_str = s_ucode.encode ("utf-8")
                if aimethod:
                    aimeth=aimethod
                    if aimeth.index('func'):
                        aimethobj= json.loads(aimeth)
                        if 'func' in aimeth:
                            #print("ai method:",aimethobj)
                            aimeth=aimethobj["func"]
            elif content['error_code']!=0:
               print(( '错误玛: ' , content['error_code']))
               print(( '错误：',content['error_msg'].encode ("utf-8")))
        return aistr, aimeth

    def urlRequest(self,url,paramData):
        pjd = json.dumps (paramData)
        response = urllib3.PoolManager ().request ("POST", url, body=pjd.encode (), headers={'Content-Type': 'application/json'})
        return json.loads (response.data)

    def acd(self, con):
        print ("join acd function")
        # con.execute('answer', '', self.uuid)
        con.execute ('set', 'tts_engine=tts_commandline', self.uuid)
        con.execute ('set', 'tts_voice=Ting-Ting', self.uuid)
        # con.execute('set', 'continue_on_fail=true', self.uuid)  # 设置这个变量的意义在于,用于在呼叫坐席失败时,不挂断电话
        con.execute ('set', 'hangup_after_bridge=true', self.uuid)  # 设置这个变量的意义在于,在于成功bridge后(即成功接通之后)坐席首先挂机,则挂断电话
        # con.execute('speak', '您好,欢迎致电,电话接通中,请稍候', self.uuid)  # 在播放完欢迎的声音之后,继续播放音乐,直到转到坐席

        con.execute ('playback', '/workspace/BeierVoice/record/beieronline/music.mp3', self.uuid)  # 播放一段音频直到电话接通
        time.sleep (50)  # 延时2s
        dial = 'user/' + '1001'  # self.FindAvailableAgent()
        print ("dial:", dial)
        con.execute ('bridge', dial, self.uuid)  # 桥接到找到的坐席

    def  get_playback_str(self,path,records):
        r_str=None
        for  record in records:
            if r_str:
                if record in 'money':
                    tts = '%s%s%s' % (
                    "shout://tsn.baidu.com/text2audio?lan=zh&ctp=1&cuid=abcdxxx&tok=" + BotTcpHandler.access_token + "&tex=",
                    self.money+"元", "&vol=9&per=0&spd=6&pit=3")
                    r_str += "@" + tts
                elif record == 'name':
                    tts = '%s%s%s' % (
                    "shout://tsn.baidu.com/text2audio?lan=zh&ctp=1&cuid=abcdxxx&tok=" + BotTcpHandler.access_token + "&tex=",
                    self.c_name+'吗？', "&vol=9&per=0&spd=6&pit=3")
                    r_str+="@"+tts
                else:
                    r_str+="@"+path+record+".mp3"
            else:
                r_str = path+record+".mp3"
        return  r_str


    def combine_record(self,record):
        if record =="money":
            pass
        elif  record=='alias':
            pass
        elif  record=='over_time':
            pass





#coding:utf-8
import time
import json
import base64
import threading
import websocket
import sys,urllib3,urllib

import AutoCall.Dialer

import  queue
from urllib3.exceptions import InsecureRequestWarning

urllib3.disable_warnings(InsecureRequestWarning)

def webRequest(method,url,body="",auth="Basic"):
    print ("task", url, body)
    try:
        if  body:
            response = urllib3.PoolManager ().request (method, url,body = body.encode (),headers={'Content-Type': 'application/json','Authorization':auth})
        else:
            response = urllib3.PoolManager ().request(method, url,headers={'Content-Type': 'application/json','Authorization':auth })
        if response.status==200:
            return json.loads (response.data)
    except ConnectionRefusedError:
        print ('服务器拒绝连接:', url)
    return None

'''
获取拨打列表
'''
def scan(isReady,**kwargs):
    if  kwargs["rest_type"] =="java":
        url = kwargs["tasks_url"] + "ready" if isReady else "excuting"
        data = webRequest ("GET", url, auth=getAuthStr (**kwargs))
        if data:
            return json.loads(json.dumps(data).lower())
    else:
        url=kwargs["tasks_url"]+  "?search=ready" if  isReady else   "?search=excuting"
        data = webRequest ("GET", url, auth=getAuthStr (**kwargs))
        if data:
            return  json.loads(json.dumps(data["results"]).lower())
    return None

def getAuthStr(**kwargs):
    upwd = kwargs['username'] + ":" + kwargs['password']
    return  "Basic "+ base64.standard_b64encode(upwd.encode()).decode()

def getDefualtLine(task):
    if task['bot']:
        pass
    if task['line']:
        line=task["line"]
        return  {'linename':line['name'], 'aiPhoneNum': '6666', 'aiDialPlanName': 'default', 'serverIp': '114.116.39.18', 'serverPort': '8021', 'clientName': 'ClueCon', 'scencenumber': '9720', 'scencename': 'IOU2345'}
    if task['scence']:
        pass
    return  {'linename':'yuneasy0794','aiPhoneNum':'6666','aiDialPlanName':'default','serverIp':'114.116.39.18','serverPort':'8021','clientName':'ClueCon','scencenumber':'9720','scencename':'IOU2345'}

def calling(**kwargs):
     task_number=0
     isReady=True
     while True:
         try:
             task_number += 1
             tasks = scan (isReady, **kwargs)
             if tasks and  len(tasks)>0:
                 for task in tasks:
                     dialer = AutoCall.Dialer.PhoneDialer (task, getDefualtLine (task), None, **kwargs)
                     tThread = threading.Thread (target=dialer.dials)
                     tThread.start ()
                     print (task['name'], 'started. index is :', task_number)
         except Exception as e:
             print ('Exception in calling:', e)
         time.sleep(120)

def config(conf_profile):
    print ('load cc config')
    with open(conf_profile,'r') as conff:
        return  json.loads(conff.read())

def run(conf_profile):
    conf = config (conf_profile)
    print ('----------------------------------------begin calling loop-------------------------------------------')
    calling(**conf)
    print('---------------------------------------------main exit!-------------------------------------------------')


#队列字典
queue_dict={}


class QueueThread(object):
    def __init__(self,tqueue,**kwargs):
        self._queue=tqueue
        self.conf=kwargs
        self.task_thread=None
        self.wait_tasks =[]

    def  run_task(self,task):
        if task["id"] not in self.wait_tasks:
            self._queue.put_nowait (task)
            self.wait_tasks.append(task["id"])
        #self.run()

    def run(self):
            if  not  (self.task_thread and  self.task_thread.is_alive()):
                if  self._queue and not self._queue.empty():
                    self.task_thread=threading.Thread(target=self.worker)
                    print('queue begin')
                    self.task_thread.start()

    def  worker(self):
        while  not self._queue.empty():
            task=self._queue.get_nowait()
            dialer = AutoCall.Dialer.PhoneDialer (task, getDefualtLine (task), None, **self.conf)
            print("task ",task['name'],"is runing")
            dialer.dials()
            print ("task ", task['name'], "is end")
        print("queue end")

def runTask(**kwargs):
    task_number = 0
    isReady = True
    scan_number = 0
    while True:
        scan_number+=1
        try:
            tasks = scan (isReady, **kwargs)
            if tasks and len (tasks) > 0:
                for task in tasks:
                    task_number += 1
                    print ("task ",task_number,':', task)
                    line_id=task["line"]["id"]
                    if line_id not in queue_dict:
                        queue_dict[line_id]=QueueThread(queue.Queue(),**kwargs)
                    queue_dict[line_id].run_task(task)
                    print (task['name'], 'started. index is :', task_number)

            if  queue_dict and len(queue_dict)>0:
                for k,v in queue_dict.items():
                    v.run()
            print("loop",scan_number)
        except Exception as e:
            print ('Exception in fetchTask:', e)
        time.sleep (60)

def  run_with_queue(conf_profile):
    conf =  config (conf_profile)
    print ('---------------------------------------------排队呼叫开启-------------------------------------------------')
    runTask (**conf)
    print ('---------------------------------------------排队呼叫结束!-------------------------------------------------')






#coding:utf-8

import ESL
import  socketserver
import json
import aip
import  xml.dom.minidom as mdom
import datetime
import time
import uuid
import urllib, urllib3, sys,urllib3
import ssl
from MyDialog.d_payback import *

class MyTCPHandler(socketserver.BaseRequestHandler):
    session_id = ''
    bot_session = ''
    # access_token = '24.11cf4eba845cde3fdea8d4c1930650f9.2592000.1526029091.282335-10938323'
    access_token = '24.3cc7f347e9ca4740cabf1164905d6d15.2592000.1529578415.282335-11126101'
    # scene_id='19157'
    scene_id ='2490' #'1821'  # '1698'#'1493'#'20775'#'20159'#
    log_id = 1;
    unit_v = 2
    count = 0   #创建一个类变量

    def __init__(self, request, client_address, server):
        super().__init__(request,client_address,server)


    def handle(self):
        print(( self.client_address,'connected!'))
        self.payback = payback ('beieronlinebak.json')
        fd= self.request.fileno()
        print (fd)
        con=ESL.ESLconnection(fd)
        con.events("json","ALL")
        #con.sendRecv('events json ALL')
        #con.sendRecv ('myevents')
        #con.sendRecv('linger')
        #con.sendRecv ('divert_event')
        self .hello="我是"

        print (( 'accept:',con.connected()))
        if con.connected():
            info = con.getInfo ()
            uuid = info.getHeader ('unique-id')
            cname = info.getHeader ('Channel-Channel-Name')
            con.execute ('set', 'fire_asr_events=true', uuid)
            con.execute ('set', "rtp_enable_vad_out=true", uuid)
            con.execute ("set", 'record_sample_rate=16000', uuid)
            self.playAndDetectSpeech(con,uuid)
            print( '准备退出')
            if  con.connected():
                #con.execute ("hangup", '', uuid)
                con.disconnect ()
                pass
            print ((cname, '已经退出'))

    def playAndDetectSpeech(self,con,uuid):

        le_sequence=0
        count=1

        isanswer=False
        canstop=False
        idlstart=None
        secondtemp = 15
        con.execute ("answer", "", uuid)
        while 1:
            e_info = con.recvEventTimed(10)
            if (not canstop) and idlstart and isanswer == True and (datetime.datetime.now() - datetime.timedelta(seconds=secondtemp) > idlstart):
                print (( '静音时段: start :',idlstart,'end:',datetime.datetime.now (),'静音连续次数：',count))
                if count==1:
                    secondtemp = 5
                    self.say (con, uuid, self.payback.fetchQA('GREET1'))
                elif count==2:
                    secondtemp = 2
                    self.say (con, uuid, self.payback.fetchQA('GREET2'))
                elif count==3:
                    secondtemp = 2
                    self.say (con, uuid, self.payback.fetchQA('GREET3'))
                elif count >= 4:
                    text = self.fetchSayTxt(con, uuid)
                    secondtemp = 2
                    if  text:
                        self.say(con, uuid, text)
                    else:
                        pass
                    #count=2
                    #secondtemp = 15
                count+=1
                idlstart=datetime.datetime.now ()
                #print  'idlestart in begin:', idlstart
            if e_info:
                ename=e_info.getHeader("Event-Name")
                e_sequence =e_info.getHeader("Event-Sequence")
                #print(ename)
                if ename=="CHANNEL_ANSWER":
                    con.execute ('detect_speech',
                                 'unimrcp {start-input-timers=false,no-input-timeout=1000}yes_no yes_no', uuid)
                    #con.execute ("playback", "/root/workspace/zw_project/VoiceBot/record/beieronline/f0lll1.mp3", uuid)
                    self.say (con, uuid, self.payback.context.current_dn)
                    isanswer = True
                if  ename== 'PLAYBACK_STOP':
                    print("p stop")
                    canstop=False
                    con.execute ('detect_speech', 'resume', uuid)
                    if self.payback.context.end:
                        break
                if ename=='PLAYBACK_START':
                    print ("p start")
                    canstop=True
                    idlstart=None
                if ename=='PRIVATE_COMMAND':
                    if isanswer:
                        print(('PRIVATE_COMMAND',idlstart))
                        if not canstop:
                            idlstart = datetime.datetime.now ()
                        else :
                            idlstart =None
                        print (('PRIVATE_COMMAND new', idlstart))
                    print("secondtemp:",secondtemp)
                if ename=='DETECTED_SPEECH' and le_sequence!=e_sequence:
                    le_sequence=e_sequence
                    speechType = e_info.getHeader ("Speech-Type")
                    self.log_id+=1
                    if speechType=='begin-speaking':
                        print( '我听到有人在说话:')
                        #con.execute ('uuid_break', uuid, uuid)
                    elif speechType == 'detected-speech':
                        idlstart = None
                        count=4
                        recogResults=e_info.getHeader('_body')
                        if recogResults == 'Completion-Cause: 002':
                            pass
                        else:
                            if canstop:
                                con.api('uuid_break '+uuid)
                                canstop=False
                            ttext = self.parseXml (recogResults)
                            con.execute ("detect_speech", 'pause', uuid)
                            text = self.fetchSayTxt(con, uuid, ttext)
                            print(("静音",secondtemp,idlstart))
                            if (not self.payback.context.end) and text :
                                print (('你说:', ttext))
                                secondtemp = len (text['text']) / 7  # 识别到正常语音，询问间隔又设置为10s
                                if secondtemp < 5: secondtemp = 5
                                self.say (con, uuid, text,ttext)
                            elif text:
                                secondtemp = len (text['text']) / 7  # 识别到正常语音，询问间隔又设置为10s
                                if secondtemp < 5:
                                    secondtemp = 5
                                    self.say(con, uuid, text,ttext)
                            else:
                                    break
                    elif speechType == 'closed':
                        #print '语音识别被closed.'
                        break
                    else:
                        pass
                if ename=='CHANNEL_HANGUP':
                    break

    def parseXml(self,xmlStr):
        if xmlStr:
            xdom=mdom.parseString (xmlStr)
            nod= xdom.documentElement.getElementsByTagName('input')
            print (nod[0].childNodes[0].nodeValue)
            if nod[0] and nod[0].childNodes[0] and nod[0].childNodes[0].nodeValue:
                strtext= nod[0].childNodes[0].nodeValue
            else:
                strtext='你说什么,我没有听清!'
            return strtext[0:-1]
        return "中华人民共和国"

    def fetchSayTxt(self, con, uuid, txtFrom = "next"):
        if self.unit_v==1:
            toTxt, sid = self.interop(txtFrom, self.session_id)
            self.session_id = sid
        if self.unit_v==2:
            #print  "unit调用:",
            toTxt, method = self.interop2(txtFrom,self.bot_session)
            print(("unit 返回结果:（对话内容）", toTxt,"|（内部方法）",method))
            if toTxt == '您要问的是以下哪个问题？' :
                toTxt = self.payback.fetchNext ("WHAT")
            elif method:
                toTxt = self.payback.fetchNext (method)
            elif toTxt :
                if toTxt=="AN1":
                    toTxt =self.payback.fetchQA('next')
                else:
                    toTxt =self.payback.fetchQA(toTxt)
            else:
                toTxt=None
        return toTxt

    def recordVoice(self,con,uuid,voiceNumber):
        #print  '开始录音'
        con.execute("record", '/usr/local/freeswitch/recordings/archive/' + str(voiceNumber) + '.wav 20 2000 2', uuid)
        # con.execute ("record_session", '/usr/local/freeswitch/recordings/archive/' + uuid + '.wav', uuid)

    def setChannelVerb(self,con,uuid):
        con.execute ("set", 'record_sample_rate=44100', uuid)
        # con.execute ("set", 'vad=out', uuid)

    def say(self,con,uuid,text,utext=""):
        print((  '机器说:', text['text']))
        if text["type"]=="tts":
            self.payback.context.appendDialogContext(utext,text)
            #tts_str_bd ='%s%s%s' % ("shout://tsn.baidu.com/text2audio?lan=zh&ctp=1&cuid=abcdxxx&tok=24.11cf4eba845cde3fdea8d4c1930650f9.2592000.1526029091.282335-10938323&tex=" , text, "&vol=9&per=0&spd=5&pit=5")
            playstr ='%s%s%s' % ("shout://tsn.baidu.com/text2audio?lan=zh&ctp=1&cuid=abcdxxx&tok="+self.access_token+"&tex=" , text['text'].replace(" ",""), "&vol=9&per=0&spd=6&pit=3")
            con.execute ("playback", playstr, uuid)
            #print "tts:",tts_str_bd
        elif text["type"]=="record":
            playstr="/root/workspace/zw_project/VoiceBot/record/beieronline/"+text["recordNumber"]+".mp3"
            print(playstr)
            con.execute ("playback", playstr, uuid)

    def interop2(self,text,bot_session,userid='mifeng'):
        if text == "next":
            return '', text
        url='https://aip.baidubce.com/rpc/2.0/unit/bot/chat?access_token=' + self.access_token
        client_session={ "client_results":"", "candidate_options":[]}
        request={
            "bernard_level":0,
            "query":text,
            "updates":"",
            "user_id":userid,
            "query_info":{"asr_candidates":[],"source":"KEYBOARD","type":"TEXT"},
            "client_session":json.dumps(client_session)
        }
        pd={'bot_session':self.bot_session,'log_id':self.log_id,'request':request,"bot_id":self.scene_id,"version":"2.0"}
        print ('bot s:',pd)
        content =self.urlRequest(url,pd)
        #print  content
        if (content):
            if 'error_code' in  content and content['error_code']==0:
                #self.bot_session = content['result']['bot_session']
                self.bot_session='';
                s_ucode=content['result']['response']['action_list'][0]['say']
                method=content['result']['response']['action_list'][0]['custom_reply']
                if s_ucode:
                    s_str = s_ucode
                    #s_str = s_ucode.encode ("utf-8")
                else:
                    s_str=''
                if method:
                   s_m_str=method
                else:
                    s_m_str=""
                return s_str,s_m_str
            elif content['error_code']!=0:
               print(( '错误玛: ' , content['error_code']))
               print(( '错误：',content['error_msg'].encode ("utf-8")))
        return '您说什么,没听清?', ''

    def interop(self,text,session_id):
        #host = 'https://aip.baidubce.com/oauth/2.0/token?grant_type=abcdexxx&client_id=s4eB21vTQssWoYlBd8jTAHVQ&client_secret=HOcBpqXuHeoGe16Gt13TQ0eUUGR8MsOs '
        url = 'https://aip.baidubce.com/rpc/2.0/solution/v1/unit_utterance?access_token=' + self.access_token
        pd={'scene_id':self.scene_id,'query':text,'session_id':session_id}
        #print '对话', pd
        content =self.urlRequest(url,pd)
        #print 'unit 返回结果：',content
        if (content):
            if 'result' in content and content['result'] and content['result']['qu_res']:
               # print  content['result']['action_list']
                s_ucode = content['result']['action_list'][0]['say']
                s_str=s_ucode.encode("utf-8")
                self.session_id = content['result']['session_id']
                #print  s_str,s_ucode
                return  s_str,self.session_id
        return  '您说什么,没听清?',''

    def urlRequest(self,url,paramData):
        pjd = json.dumps (paramData)
       # print (pjd)
        response = urllib3.PoolManager ().request ("POST", url, body=pjd.encode (), headers={'Content-Type': 'application/json'})
        #print (response.data)
        return json.loads (response.data)

server=socketserver.ThreadingTCPServer(("",8040),MyTCPHandler)
server.serve_forever()




#coding:utf-8

import json


class payback():
    def __init__(self,filename='payback.json',**kwargs):
        self.context = context (filename, 'r',**kwargs)
        print(("话术加载完毕",self.context))
        '''
        self.dict = {
                     'hello':self.hello,
                     'nohear': self.nohear, 'default_next': self.current, 'yes': self.yes,
                     'answer_yes': self.answer_yes, 'answer_no': self.answer_no, 'payback_yes': self.payback_yes,
                     'payback_no': self.payback_no, 'unknow': self.unknow, 'other': self.other, 'bye': self.bye}
        '''
    def fetchNext(self,func):
       #print (("dict-v:",self.dict.keys()))
        return self.context.fetch(func)
    def fetchQA(self,qestionNum):
        return self.context.fetchQA(qestionNum)


class context():
    def __init__(self,filename,fype,**kwargs):
        self.dialogContexts=None
        self._end=False
        self.dialog=None
        self.qestion_annswer=None
        self.score=0
        self.attitude=['92','无还款诚意']
        self.timesCtrl={'nohear':0,'what':0,'an2':0}
        if kwargs:
            self.dialog =kwargs
            self.current_dn = self.dialog['flow']
            self.qestion_annswer = self.dialog['qestionAnnswers']
        else:
            if fype=='r':
                self.load (filename)
            else:
                self.current_dn = None
                self.save (filename)

    def load(self,fileName):
        with open (fileName, 'r') as load_f:
            print ( '读取json话术')
            self.dialog = json.load (load_f)
            self.current_dn=self.dialog['flow']
            #print (("flow:", self.current_dn))
            self.qestion_annswer=self.dialog['qestionAnnswers']
            #print (("qestionAnnswers:",self.qestion_annswer))

    def save(self,fileName):
        with open (fileName, 'w') as dump_f:
            print  ( '保存json话术')
            json.dump (self.current_dn,dump_f)

    '''
    def fetch(self,which='next'):
        #print "current:", self.current_dn,type(self.current_dn)
        temp = None
        if self.current_dn:
            if ( which=='yes'):
                if 'left' in self.current_dn:
                    temp=self.current_dn["left"]
                else:
                    if  'next' in self.current_dn:
                        temp=self.current_dn["next"]
            elif which=='no':
                if 'right' in self.current_dn:
                    temp=self.current_dn["right"]
                else:
                    if  'next' in self.current_dn:
                        temp=self.current_dn["next"]
            elif which == "next" and  'next' in self.current_dn:
                temp = self.current_dn["next"]
            elif which == "nohear":
                dcontexts =self.getDialogContexts()
                aisaysLen=len(dcontexts)
                if  aisaysLen>0:
                    return dcontexts[aisaysLen-1]["ai"]
            elif which=='bye':
                self.end=True
            else  :#如果无法识别就走next
                if 'next' in self.current_dn :
                    temp = self.current_dn["next"]
            if temp and (not ( "left" in temp or "right" in temp or "next" in temp)):
                self.end=True
        if  temp :
            self.current_dn=temp
        return  temp
    '''

    @property
    def  end(self):
        if   self.current_dn and  'isEnd' in self.current_dn:
            return  self.current_dn['isEnd'].lower()=='true' or self._end
        return  self._end

    def check_over_times(self,position):
        if position == 'nohear':
            if  self.timesCtrl[position]>2:#超过两次走主流,人没听清，重复上一句
                self.timesCtrl[position] = 0
                return  'next'
            else:
                self.timesCtrl['nohear'] += 1
                return  position
        elif  position == 'an2':
            if  self.timesCtrl[position]>2:#超过两次走结束,不好意思，您刚说什么啊
                return  'AN2end'
            else:
                self.timesCtrl['an2'] += 1
                return  position
        elif position == 'what':
            if self.timesCtrl[position] >2:#超过两次走结束,您说什么,我没听清楚,能再说一遍吗？
                return 'WHATend'
            else:
                self.timesCtrl['what'] += 1
                return  position
        elif position=='no':
            return  'right'
        elif position=='yes':
            return  'left'
        self.timesCtrl[position]=0
        return  position

    def get_sub_node(self,position):
        v_position=self.check_over_times(position)
        if  v_position =='nohear':
            dcontexts = self.getDialogContexts()
            aisaysLen = len(dcontexts)
            if aisaysLen > 0:
                tnode= dcontexts[aisaysLen - 1]["ai"]
            else:
                tnode=None
        else:
            tnode=self.current_dn[v_position] if v_position in self.current_dn else None
            if  (position=='yes' or   position=='no') and  tnode is  None:
                tnode = self.current_dn['next'] if 'next' in self.current_dn else None
        return tnode

    def  fetch(self,query='next'):
        queryR=None
        if  self.current_dn:#话术正确加载，当前节点明确
            if  query.lower()=='bye':
                self._end=True
            else:
                queryR = self.get_sub_node(query.lower())
        if  queryR :
            self.current_dn=queryR
         #话术未加载或者加载失败,或者通话要结束了,当前节点不明确
        return  queryR

    def fetchQA(self,qestionNum):
        if self.qestion_annswer and len(self.qestion_annswer) and qestionNum in self.qestion_annswer:
            return  self.qestion_annswer[qestionNum]
        return  None

    def appendDialogContext(self,aisay,mansay=""):
        if not self.dialogContexts:
            self.dialogContexts=[]
        self.score+=self.caculateScore (mansay)
        self.attitude=self.check_attitude(aisay)
        self.dialogContexts.append({"man":mansay,"ai":aisay})

    def check_attitude(self,ai):
       # print(type(ai),ai)
        if  'score' in ai :
            for k, v in ai["score"].items():
                if  k =='attitude':
                    return  v
        return  self.attitude

    '''
    评分应该取电脑上一句说的，因为人说的话是对电脑上一句的回答
    '''
    def caculateScore(self,man):
        if len(self.dialogContexts)>0 :
            ai=self.dialogContexts[-1]
            if "ai" in ai and "score" in ai["ai"]:
                for k,v in ai["ai"]["score"].items():
                    if k in man or man in k:
                        return  v
        return  0

    def getDialogContexts(self):
        if not self.dialogContexts:
            self.dialogContexts=[]
        return  self.dialogContexts




#coding:utf-8

import ESL
import socketserver



def dail(con,number):
    #9720    2490
    #dail_str='originate user/'+number+' &echo'
    #dail_str='originate sofia/gateway/yuneasy0794/15062405910 &playback /usr/local/freeswitch/recordings/archive/test.wav'
    #dail_str = 'originate user/1005 ai01 xml default'
    #dail_str = 'originate 8888 &bridge(user/1000)'
    # 13772491757  祝  13916483739 张 15062405910,17321307255  李 18209288679
    dail_str='originate {originattion_caller_id_number=7777,Scence_id=9720,Scence_name=yundaozichan,callee_id_number='+number+',config_name=testbot}sofia/gateway/yuneasy0794/'+number+' 7777 xml default'
    #dail_str = 'originate  sofia/gateway/yuneasy0794/' + number+' &socket(127.0.0.1:8040 async full)'
    #dail_str='originate sofia/gateway/yuneasy0794/'+number+' socket:192.168.3.146:8040 async  full'zhongzilian

    #dail_str = 'originate user/1000 8888 xml default'
    print(dail_str)
    return con.api(dail_str)

def autoDail():
    con =ESL.ESLconnection('114.116.39.18','8021','ClueCon')
    if con.connected():
        print ( '已链接上')
        con.events('json','all')
        evt = dail (con, '13772491757')
        while 1:
            evt =con.recvEvent()
            if evt :
                evt_name=evt.getHeader("Event-Name")
                print (("event :", evt_name))
                print(evt.serialize ('json'))
                if evt_name=="SERVER_DISCONNECTED" or evt_name=="CHANNEL_DESTROY":
                    break
                if evt_name=='CHANNEL_ANSWER':
                    pass
                if  evt_name=='CHANNEL_HANGUP':
                    print ()
                    break
                if  evt_name=='CHANNEL_STATE'  or evt_name=='CHANNEL_CALLSTATE':
                    pass
                 #print (evt.serialize ('json'))


    con.disconnect()
    #print '已断开链接'

if __name__ == '__main__':
    autoDail()
    print('结束拨号过程')
